// Orbital Shaker Code
// Updated May 10, 2022
// Written by Lukas Scheidl and Jay-Young Cho

// Include the necessary libraries:
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <AccelStepper.h>

// Defining LCD Module
LiquidCrystal_I2C lcd(0x27, 16, 2);  // sets the lcd address to 0x27

// Define stepper motor connections and motor interface type. Motor interface type must be set to 1 when using a driver:
#define dirPin 2
#define stepPin 3
#define motorInterfaceType 1

// Create a new instance of the AccelStepper class:
AccelStepper stepper = AccelStepper(motorInterfaceType, stepPin, dirPin);

// setting lower and upper limits of rpm (MANUAL USER INPUT)
int lower = 120;
int upper = 300;
int range = upper - lower;

// Defining curNum for screen clearing function within loop
int num;
int curNum = 0;

// Defining rpm value to be chosen by user with potentiometer and translated into steps for our motor to accelerate to
int rpm;

void setup() {
  Serial.begin(9600);  //initialize serial communication at 9600 bits per second

  // Initial message
  lcd.init();
  lcd.backlight();  // note that it will work even with the backlight off
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Welcome to Orbi!");
  lcd.setCursor(1, 1);
  lcd.print("by KC,JC,LS,SZ");
  delay(3000);

  // Message to input potentiometer rpm
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("You have 10 sec");
  lcd.setCursor(2, 1);
  lcd.print("to input RPM");
  delay(3000);
  lcd.clear();

  // Message to output current range
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("RPM Range:");
  lcd.setCursor(4, 1);
  String lowText = String(lower);
  String highText = String(upper);
  String rangeText = lowText + "-" + highText;
  lcd.print(rangeText);
  delay(3000);
  lcd.clear();

  // Loop to set the rpm
  int starttime = millis();
  int endtime = starttime;
  int loopcount = 0;
  while ((endtime - starttime) <= 10000) // do this loop for up to 10000mS = 10S
  {
    float sensorVal = analogRead(A0);                 //read the input on analog pin 0
    float rpmRaw = sensorVal / 1023 * range + lower;  // raw RPM value as a float

    int rpmSet;
    int multByTen = ((int)rpmRaw / 10) + 1;  // since (int) truncates, adding 1 rounds up each value
    rpmSet = 10 * multByTen;
    rpmSet = rpmSet - 10;  // correction because 2 lines above, everything is rounded up by 10

    // Loop to clear the screen if new value has fewer values than the previous value
    String val = String(rpmSet);
    int num = val.length();

    if (num < curNum) {
      lcd.clear();
    }
    curNum = num;

    lcd.setCursor(1, 0);
    lcd.print("Orbital Shaker");
    lcd.setCursor(3, 1);
    lcd.print("RPM = " + String(rpmSet));

    rpm = rpmSet;
    
    loopcount = loopcount+1;
    endtime = millis();
  }

  // setting the stepper speed and acceleration
  int setSpeed = rpm / 60 * 200;
  stepper.setMaxSpeed(setSpeed);
  stepper.setAcceleration(60); // 40 steps per second per second
  stepper.moveTo(100000000);
}

void loop() {
  // for (int i = 1; i < 8000; i++) {
    stepper.runToPosition();
    stepper.run();
  //  }
   
  // Serial.println(stepper.speed());
}
